==> jekyll-images-0.2.1.gem: lib/jekyll/images/thumbnail.rb <==
# frozen_string_literal: true

require 'vips'

module Jekyll
  module Images
    # Use LibVIPS to generate images in different sizes
    #
    # We're assuming every image is going to be thumbnailed
    class Thumbnail
      attr_reader :site, :image, :thumbnail, :filename, :width, :height

      def initialize(site, filename, **args)
        unless File.exist? filename
          raise ArgumentError, "File not found: #{filename}"
        end
        raise ArgumentError, 'Missing width' unless args[:width]

        @site = site
        @filename = filename
        @width = args[:width]
        @height = args[:height]
        @image = Vips::Image.new_from_file filename, access: :sequential
        @thumbnail = image.thumbnail_image width,
                                           height: height || proportional_height,
                                           auto_rotate: args[:auto_rotate],
                                           crop: args[:crop].to_sym
      end

      # Finds a height that's proportional to the width
      def proportional_height
        @height = (image.height * (width / image.width.to_f)).round
      end

      # Generates a destination from filename only if we're downsizing
      def dest
        @dest ||= if image.width > width
                    filename.gsub(/#{extname}\z/, "_#{width}x#{height}#{extname}")
                  else
                    Jekyll.logger.info "Not thumbnailing #{filename}"
                    filename
                  end
      end

      def extname
        @extname ||= File.extname filename
      end

      # Only write when the source is newer than the thumbnail
      def write?
        !File.exist?(dest) || File.mtime(filename) > File.mtime(dest)
      end

      # Save the file into destination if needed and add to files to
      # copy
      def write
        return unless write?

        Jekyll.logger.info "Thumbnailing #{filename} => #{dest}"
        thumbnail.write_to_file(dest)

        # Add it to the static files so Jekyll copies them.  Once they
        # are written they're copied when the site is loaded.
        site.static_files << Jekyll::StaticFile.new(site, site.source,
                                                    File.dirname(dest),
                                                    File.basename(dest))

        # The file was updated, so it exists and is newer than source
        !write?
      end

      # Run optimizations
      def optimize
        before, after = Runner.run(dest)

        return unless before

        pct = ((after.to_f / before) * -100 + 100).round(2)

        Jekyll.logger.info "Reduced #{filename} from #{before} to #{after} bytes (%#{pct})"
      end
    end
  end
end

==> jekyll-images-0.2.4.gem: lib/jekyll/images/thumbnail.rb <==
# frozen_string_literal: true

require 'vips'

module Jekyll
  module Images
    # Use LibVIPS to generate images in different sizes
    #
    # We're assuming every image is going to be thumbnailed
    class Thumbnail
      attr_reader :site, :filename, :width, :height, :crop, :auto_rotate

      def initialize(site, filename, **args)
        unless File.exist? filename
          raise ArgumentError, "File not found: #{filename}"
        end
        raise ArgumentError, 'Missing width' unless args[:width]

        @site = site
        @filename = filename
        @width = args[:width]
        @height = args[:height]
        @crop = args[:crop].to_sym
        @auto_rotate = args[:auto_rotate]
      end

      def image
        @image ||= Vips::Image.new_from_file filename,
                                             access: :sequential
      end

      def thumbnail
        @thumbnail ||= image.thumbnail_image width,
                                             height: height,
                                             auto_rotate: auto_rotate,
                                             crop: crop
      end

      def height
        @height || proportional_height
      end

      # Finds a heigth that's proportional to the width
      def proportional_height
        @proportional_height ||= (image.height * (width / image.width.to_f)).round
      end

      # Generates a destination from filename only if we're downsizing
      def dest
        @dest ||= if image.width > width
                    filename.gsub(/#{extname}\z/, "_#{width}x#{height}#{extname}")
                  else
                    Jekyll.logger.info "Not thumbnailing #{filename}"
                    filename
                  end
      end

      def extname
        @extname ||= File.extname filename
      end

      # Only write when the source is newer than the thumbnail
      def write?
        !File.exist?(dest) || File.mtime(filename) > File.mtime(dest)
      end

      # Save the file into destination if needed and add to files to
      # copy
      def write
        return unless write?

        Jekyll.logger.info "Thumbnailing #{filename} => #{dest}"
        thumbnail.write_to_file(dest)

        # Add it to the static files so Jekyll copies them.  Once they
        # are written they're copied when the site is loaded.
        site.static_files << Jekyll::StaticFile.new(site, site.source,
                                                    File.dirname(dest),
                                                    File.basename(dest))

        # The file was updated, so it exists and is newer than source
        !write?
      end

      # Run optimizations
      def optimize
        before, after = Runner.run(dest)

        return unless before

        pct = ((after.to_f / before) * -100 + 100).round(2)

        Jekyll.logger.info "Reduced #{filename} from #{before} to #{after} bytes (%#{pct})"
      end
    end
  end
end
